# -*- coding: utf-8 -*-
"""
The coefficient functions definition is given in :eqref:`4.2`,
:cite:`vogt-f2nc` (that is the main reference for their expression, i.e. all
the formulas in this module) (the same of :eqref:`1` in :cite:`vogt-flnc`).
The main reference for their expression is :cite:`vogt-flnc`.

Scale varitions main reference is :cite:`nnlo-sv-singlet` and
:cite:`nnlo-sv-nonsinglet`.

Scale variations main reference is :cite:`moch-f3nc`.

Note
----
    Check the theory reference for details on
    :doc:`../theory/scale-variations`
"""

from . import kernels
